﻿local L = LibStub("AceLocale-3.0"):GetLocale("Critters", false)

Critters = CreateFrame("frame")
local Critters = Critters
Critters:SetScript("OnEvent", function(self, event, ...) if self[event] then return self[event](self, event, ...) end end)
Critters:RegisterEvent("ADDON_LOADED")
function Critters:Print(...) ChatFrame1:AddMessage(string.join(" ", "|cFF33FF99Critters|r:", ...)) end

local CritterDBver = 1

-- create a tooltip scanner
local Scanner = CreateFrame("GameTooltip", "Critters_ScannerTooltip", nil, "GameTooltipTemplate")
Scanner:SetOwner(UIParent, "ANCHOR_NONE")
for i = 1, 30 do
	Scanner[i] = _G["Critters_ScannerTooltipTextLeft"..i]
	if not Scanner[i] then
		Scanner[i] = Scanner:CreateFontString()
		Scanner:AddFontStrings(Scanner[i], Scanner:CreateFontString())
	end
end

function Critters:ADDON_LOADED(event, addon)
	if addon ~= "Critters" then return end
	local _, class = UnitClass("player")
	for k, macro in pairs (self.macros[class]) do
		self.macros['GENERAL'][k] = macro
	end
	self.locked = {}
	for k in pairs (self.macros['GENERAL']) do
		self.locked[k] = true
	end
	defaults = {
		['class'] = {
			cm = 'Default',
		},
		['char'] = {
			cmloc = 'class',
			cmenabled = false,
			Critters = {},
		},
		['global'] = {
			['macros'] = self.macros['GENERAL'],
		}
	}
	self.db = LibStub("AceDB-3.0"):New("CrittersDB", defaults, 'class')
	if self.db.char.CritterDBver ~= CritterDBver then
		self.db.char.CritterDBver = CritterDBver
		self.db.char.Critters = {}
	end
	self:UnregisterEvent("ADDON_LOADED")
	self.ADDON_LOADED = nil
	if IsLoggedIn() then self:PLAYER_LOGIN() else self:RegisterEvent("PLAYER_LOGIN") end
	
	-- register SlashCommand
	SLASH_Critters1 = "/Critters";
	SlashCmdList["Critters"] = function()
		InterfaceOptionsFrame_OpenToCategory(self.optframe.Critters)
	end
end

function Critters:PLAYER_LOGIN()
	if GetMacroIndexByName("Critters") == 0 then
		CreateMacro("Critters", 1, '')
	end
	self:Scan()
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("COMPANION_LEARNED")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self.PLAYER_LOGIN = nil
	self:UpdateOptions()
end

function Critters:PLAYER_REGEN_DISABLED()
	self:UpdateMacro(true)
end

function Critters:PLAYER_REGEN_ENABLED()
	self:UpdateMacro()
end

function Critters:COMPANION_LEARNED()
	self:Scan()
end

function Critters:UNIT_SPELLCAST_SUCCEEDED(event, arg1, arg2)
	if arg1 == 'player' and self.db.char.Critters[self.ReverseLookup[arg2]] then
		self:UpdateMacro()
	end
end

function Critters:UpdateMacro(combat)
	if not InCombatLockdown() then
		if self.db.char.cmenabled and combat then
			EditMacro(GetMacroIndexByName("Critters"), "Critters", 1, self:ParseMacro(self.db.global.macros[self.db[self.db.char.cmloc].cm]))
		else
			local bugs = {}
			for k, priority in pairs(self.db.char.Critters) do
				if priority ~= 0 then
					for i=1, priority do
						bugs[#bugs+1] = k
					end
				end
			end
			local critter = bugs[math.random(1,#bugs)]
			local crittername = GetSpellInfo(critter)
			if Critters:NeedsReagents(critter) then
				Critters:UpdateMacro()
			else
				EditMacro(GetMacroIndexByName("Critters"), "Critters", 1, string.format('#showtooltip\n/cast %s()', crittername))
			end
		end
	end
end

Critters.ReverseLookup = {}
function Critters:Scan()
	local creatureSpellID
	for id = 1,GetNumCompanions("CRITTER") do
		creatureSpellID = select(3, GetCompanionInfo("CRITTER", id))
		Critters.ReverseLookup[GetSpellInfo(creatureSpellID)] = creatureSpellID
		self.db.char.Critters[creatureSpellID] = self.db.char.Critters[creatureSpellID] or 5
	end
	self:UpdateMacro()
	self:UpdateOptions()
end

function Critters:ParseMacro(macrotext)
	local spells = {}
	for mtype, mspell in string.gmatch(macrotext, "<(%a)(%w+)>") do
		if mtype == 's' then
			spells[#spells+1] = GetSpellInfo(mspell)
		end
	end
	return string.format(string.gsub(macrotext, '<%w+>', '%%s'), unpack(spells))
end

function Critters:ScanBags(itemname)
	for bagID = 0, 4 do
		for slotID = 1, GetContainerNumSlots(bagID) do
			if GetContainerItemLink(bagID, slotID) and itemname == GetItemInfo(GetContainerItemLink(bagID, slotID)) then
				return true
			end
		end
	end
	return false
end

local NeedsReagentsCache = {} -- do not cache critters that need reagets even if they are present
function Critters:NeedsReagents(critterspell)
	if NeedsReagentsCache[critterspell] then return false end
	Scanner:ClearLines()
	Scanner:SetHyperlink('spell:'..critterspell)
	for i=1,Scanner:NumLines() do
		if string.find (Scanner[i]:GetText(), SPELL_REAGENTS ,1, true) then
			if Critters:ScanBags(string.gsub(Scanner[i]:GetText(), SPELL_REAGENTS, '')) then
				return false
			else
				return true
			end
		end
	end
	NeedsReagentsCache[critterspell] = true
	return false
end
